@@ -114,12 +114,13 @@ static void dp_settings_print_usage(const char *const app,
     fprintf(stderr, "Supported options:\n"
             " --input/-i  $file:    input file\n"
             " --untimed/-u:         ignore PTS, render as fast as possible\n"
-            " --framethreads $num:  number of frame threads (default: 1)\n"
-            " --tilethreads $num:   number of tile threads (default: 1)\n"
-            " --pfthreads $num:     number of postfilter threads(default: 1)\n"
+            " --threads $num:       number of threads (default: 0)\n"
+            " --framedelay $num:    maximum frame delay, capped at $threads (default: 0);\n"
+            "                       set to 1 for low-latency decoding\n"
             " --highquality:        enable high quality rendering\n"
             " --zerocopy/-z:        enable zero copy upload path\n"
             " --gpugrain/-g:        enable GPU grain synthesis\n"
+            " --fullscreen/-f:      enable full screen mode\n"
             " --version/-v:         print version and exit\n"
             " --renderer/-r:        select renderer backend (default: auto)\n");
     exit(1);
@@ -144,12 +145,11 @@ static void dp_rd_ctx_parse_args(Dav1dPlayRenderContext *rd_ctx,
     Dav1dSettings *lib_settings = &rd_ctx->lib_settings;
 
     // Short options
-    static const char short_opts[] = "i:vuzgr:";
+    static const char short_opts[] = "i:vuzgfr:";
 
     enum {
-        ARG_FRAME_THREADS = 256,
-        ARG_TILE_THREADS,
-        ARG_POSTFILTER_THREADS,
+        ARG_THREADS = 256,
+        ARG_FRAME_DELAY,
         ARG_HIGH_QUALITY,
     };
 
@@ -158,12 +158,12 @@ static void dp_rd_ctx_parse_args(Dav1dPlayRenderContext *rd_ctx,
         { "input",          1, NULL, 'i' },
         { "version",        0, NULL, 'v' },
         { "untimed",        0, NULL, 'u' },
-        { "framethreads",   1, NULL, ARG_FRAME_THREADS },
-        { "tilethreads",    1, NULL, ARG_TILE_THREADS },
-        { "pfthreads",      1, NULL, ARG_POSTFILTER_THREADS },
+        { "threads",        1, NULL, ARG_THREADS },
+        { "framedelay",     1, NULL, ARG_FRAME_DELAY },
         { "highquality",    0, NULL, ARG_HIGH_QUALITY },
         { "zerocopy",       0, NULL, 'z' },
         { "gpugrain",       0, NULL, 'g' },
+        { "fullscreen",     0, NULL, 'f'},
         { "renderer",       0, NULL, 'r'},
         { NULL,             0, NULL, 0 },
     };
@@ -188,20 +188,19 @@ static void dp_rd_ctx_parse_args(Dav1dPlayRenderContext *rd_ctx,
             case 'g':
                 settings->gpugrain = true;
                 break;
+            case 'f':
+                settings->fullscreen = true;
+                break;
             case 'r':
                 settings->renderer_name = optarg;
                 break;
-            case ARG_FRAME_THREADS:
-                lib_settings->n_frame_threads =
-                    parse_unsigned(optarg, ARG_FRAME_THREADS, argv[0]);
+            case ARG_THREADS:
+                lib_settings->n_threads =
+                    parse_unsigned(optarg, ARG_THREADS, argv[0]);
                 break;
-            case ARG_TILE_THREADS:
-                lib_settings->n_tile_threads =
-                    parse_unsigned(optarg, ARG_TILE_THREADS, argv[0]);
-                break;
-            case ARG_POSTFILTER_THREADS:
-                lib_settings->n_postfilter_threads =
-                    parse_unsigned(optarg, ARG_POSTFILTER_THREADS, argv[0]);
+            case ARG_FRAME_DELAY:
+                lib_settings->max_frame_delay =
+                    parse_unsigned(optarg, ARG_FRAME_DELAY, argv[0]);
                 break;
             default:
                 dp_settings_print_usage(argv[0], NULL);
@@ -246,54 +245,62 @@ static Dav1dPlayRenderContext *dp_rd_ctx_create(int argc, char **argv)
         return NULL;
     }
 
-    // Register a custom event to notify our SDL main thread
-    // about new frames
-    rd_ctx->event_types = SDL_RegisterEvents(3);
-    if (rd_ctx->event_types == UINT32_MAX) {
-        fprintf(stderr, "Failure to create custom SDL event types!\n");
-        free(rd_ctx);
-        return NULL;
-    }
-
-    rd_ctx->fifo = dp_fifo_create(5);
-    if (rd_ctx->fifo == NULL) {
-        fprintf(stderr, "Failed to create FIFO for output pictures!\n");
-        free(rd_ctx);
-        return NULL;
-    }
-
-    rd_ctx->lock = SDL_CreateMutex();
-    if (rd_ctx->lock == NULL) {
-        fprintf(stderr, "SDL_CreateMutex failed: %s\n", SDL_GetError());
-        dp_fifo_destroy(rd_ctx->fifo);
-        free(rd_ctx);
-        return NULL;
-    }
-
     // Parse and validate arguments
     dav1d_default_settings(&rd_ctx->lib_settings);
     memset(&rd_ctx->settings, 0, sizeof(rd_ctx->settings));
     dp_rd_ctx_parse_args(rd_ctx, argc, argv);
 
+    // Init SDL2 library
+    if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_TIMER) < 0) {
+        fprintf(stderr, "SDL_Init failed: %s\n", SDL_GetError());
+        goto fail;
+    }
+
+    // Register a custom event to notify our SDL main thread
+    // about new frames
+    rd_ctx->event_types = SDL_RegisterEvents(3);
+    if (rd_ctx->event_types == UINT32_MAX) {
+        fprintf(stderr, "Failure to create custom SDL event types!\n");
+        goto fail;
+    }
+
+    rd_ctx->fifo = dp_fifo_create(5);
+    if (rd_ctx->fifo == NULL) {
+        fprintf(stderr, "Failed to create FIFO for output pictures!\n");
+        goto fail;
+    }
+
+    rd_ctx->lock = SDL_CreateMutex();
+    if (rd_ctx->lock == NULL) {
+        fprintf(stderr, "SDL_CreateMutex failed: %s\n", SDL_GetError());
+        goto fail;
+    }
+
     // Select renderer
     renderer_info = dp_get_renderer(rd_ctx->settings.renderer_name);
 
     if (renderer_info == NULL) {
-        printf("No suitable rendered matching %s found.\n",
+        printf("No suitable renderer matching %s found.\n",
             (rd_ctx->settings.renderer_name) ? rd_ctx->settings.renderer_name : "auto");
     } else {
         printf("Using %s renderer\n", renderer_info->name);
     }
 
-    rd_ctx->rd_priv = (renderer_info) ? renderer_info->create_renderer() : NULL;
+    rd_ctx->rd_priv = (renderer_info) ? renderer_info->create_renderer(&rd_ctx->settings) : NULL;
     if (rd_ctx->rd_priv == NULL) {
-        SDL_DestroyMutex(rd_ctx->lock);
-        dp_fifo_destroy(rd_ctx->fifo);
-        free(rd_ctx);
-        return NULL;
+        goto fail;
     }
 
     return rd_ctx;
+
+fail:
+    if (rd_ctx->lock)
+        SDL_DestroyMutex(rd_ctx->lock);
+    if (rd_ctx->fifo)
+        dp_fifo_destroy(rd_ctx->fifo);
+    free(rd_ctx);
+    SDL_Quit();
+    return NULL;
 }
 
 /**
@@ -668,10 +675,6 @@ int main(int argc, char **argv)
         return 1;
     }
 
-    // Init SDL2 library
-    if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_TIMER) < 0)
-        return 10;
-
     // Create render context
     Dav1dPlayRenderContext *rd_ctx = dp_rd_ctx_create(argc, argv);
     if (rd_ctx == NULL) {
@@ -717,9 +720,7 @@ int main(int argc, char **argv)
             if (e->type == SDL_QUIT) {
                 dp_rd_ctx_request_shutdown(rd_ctx);
                 dp_fifo_flush(rd_ctx->fifo, destroy_pic);
-                SDL_FlushEvent(rd_ctx->event_types + DAV1D_EVENT_NEW_FRAME);
-                SDL_FlushEvent(rd_ctx->event_types + DAV1D_EVENT_SEEK_FRAME);
-                num_frame_events = 0;
+                goto out;
             } else if (e->type == SDL_WINDOWEVENT) {
                 if (e->window.event == SDL_WINDOWEVENT_SIZE_CHANGED) {
                     // TODO: Handle window resizes
@@ -730,6 +731,10 @@ int main(int argc, char **argv)
                 SDL_KeyboardEvent *kbde = (SDL_KeyboardEvent *)e;
                 if (kbde->keysym.sym == SDLK_SPACE) {
                     dp_rd_ctx_toggle_pause(rd_ctx);
+                } else if (kbde->keysym.sym == SDLK_ESCAPE) {
+                    dp_rd_ctx_request_shutdown(rd_ctx);
+                    dp_fifo_flush(rd_ctx->fifo, destroy_pic);
+                    goto out;
                 } else if (kbde->keysym.sym == SDLK_LEFT ||
                            kbde->keysym.sym == SDLK_RIGHT)
                 {
@@ -782,5 +787,6 @@ out:;
     int decoder_ret = 0;
     SDL_WaitThread(decoder_thread, &decoder_ret);
     dp_rd_ctx_destroy(rd_ctx);
+    SDL_Quit();
     return decoder_ret;
 }
