@@ -32,19 +32,17 @@
 #include "src/cpu.h"
 #include "src/riscv/cpu.h"
 
-#if HAVE_GETAUXVAL
+#if HAVE_GETAUXVAL || HAVE_ELF_AUX_INFO
 #include <sys/auxv.h>
-
 #define HWCAP_RVV (1 << ('v' - 'a'))
-
 #endif
 
 int dav1d_has_compliant_rvv(void);
 
 COLD unsigned dav1d_get_cpu_flags_riscv(void) {
     unsigned flags = dav1d_get_default_cpu_flags();
-#if HAVE_GETAUXVAL
-    unsigned long hw_cap = getauxval(AT_HWCAP);
+#if HAVE_GETAUXVAL || HAVE_ELF_AUX_INFO
+    unsigned long hw_cap = dav1d_getauxval(AT_HWCAP);
     flags |= (hw_cap & HWCAP_RVV) && dav1d_has_compliant_rvv() ? DAV1D_RISCV_CPU_FLAG_V : 0;
 #endif
 
