@@ -32,6 +32,7 @@
 
 #include "common/attributes.h"
 
+#include "src/cpu.h"
 #include "src/x86/cpu.h"
 
 typedef struct {
@@ -52,12 +53,11 @@ COLD unsigned dav1d_get_cpu_flags_x86(void) {
         };
     } cpu;
     dav1d_cpu_cpuid(&cpu.r, 0, 0);
-    unsigned flags = 0;
+    unsigned flags = dav1d_get_default_cpu_flags();
 
     if (cpu.max_leaf >= 1) {
         CpuidRegisters r;
         dav1d_cpu_cpuid(&r, 1, 0);
-        const unsigned model  = ((r.eax >> 4) & 0x0f) + ((r.eax >> 12) & 0xf0);
         const unsigned family = ((r.eax >> 8) & 0x0f) + ((r.eax >> 20) & 0xff);
 
         if (X(r.edx, 0x06008000)) /* CMOV/SSE/SSE2 */ {
@@ -87,10 +87,8 @@ COLD unsigned dav1d_get_cpu_flags_x86(void) {
         }
 #endif
         if (!memcmp(cpu.vendor, "AuthenticAMD", sizeof(cpu.vendor))) {
-            if ((flags & DAV1D_X86_CPU_FLAG_AVX2) && (family < 0x19 ||
-                (family == 0x19 && (model < 0x10 || (model >= 0x20 && model < 0x60)))))
-            {
-                /* Excavator, Zen, Zen+, Zen 2, Zen 3, Zen 3+ */
+            if ((flags & DAV1D_X86_CPU_FLAG_AVX2) && family <= 0x19) {
+                /* Excavator, Zen, Zen+, Zen 2, Zen 3, Zen 3+, Zen 4 */
                 flags |= DAV1D_X86_CPU_FLAG_SLOW_GATHER;
             }
         }
